@@ -1,7 +1,7 @@
-use crate::packet::v5::extension_fields::{ReferenceIdRequest, ReferenceIdResponse};
 use crate::packet::v5::NtpClientCookie;
+use crate::packet::v5::extension_fields::{ReferenceIdRequest, ReferenceIdResponse};
 use rand::distributions::{Distribution, Standard};
-use rand::{thread_rng, Rng};
+use rand::{Rng, thread_rng};
 use std::array::from_fn;
 use std::fmt::{Debug, Formatter};
 
@@ -51,7 +51,7 @@ impl ServerId {
         // FIXME: sort IDs so we access the filters predictably
         // FIXME: check for double rolls to reduce false positive rate
 
-        Self(from_fn(|_| rng.gen()))
+        Self(from_fn(|_| rng.r#gen()))
     }
 }
 
@@ -104,7 +104,7 @@ impl BloomFilter {
         &self.0
     }
 
-    fn set_bit(&mut self, idx: U12) {
+    const fn set_bit(&mut self, idx: U12) {
         let (idx, mask) = idx.byte_and_mask();
         self.0[idx] |= mask;
     }
@@ -156,7 +156,7 @@ impl RemoteBloomFilter {
     /// * divide 512 without remainder
     /// * between `4..=512`
     pub const fn new(chunk_size: u16) -> Option<Self> {
-        if chunk_size % 4 != 0 {
+        if !chunk_size.is_multiple_of(4) {
             return None;
         }
 
@@ -182,7 +182,7 @@ impl RemoteBloomFilter {
         self.is_filled.then_some(&self.filter)
     }
 
-    pub fn next_request(&mut self, cookie: NtpClientCookie) -> ReferenceIdRequest {
+    pub const fn next_request(&mut self, cookie: NtpClientCookie) -> ReferenceIdRequest {
         let offset = self.next_to_request;
         let last_request = self.last_requested.replace((offset, cookie));
 
@@ -219,7 +219,7 @@ impl RemoteBloomFilter {
         Ok(())
     }
 
-    fn advance_next_to_request(&mut self) {
+    const fn advance_next_to_request(&mut self) {
         self.next_to_request = (self.next_to_request + self.chunk_size) % BloomFilter::BYTES as u16;
 
         if self.next_to_request == 0 {
@@ -380,7 +380,7 @@ mod tests {
                 continue;
             };
 
-            for _chunk in 0..((512 / chunk_size) + 1) {
+            for _chunk in 0..=(512 / chunk_size) {
                 let cookie = NtpClientCookie::new_random();
                 let request = bf.next_request(cookie);
                 let response = request.to_response(&target_filter).unwrap();
