@@ -7,11 +7,11 @@ use std::{
     time::{Duration, Instant},
 };
 
-use serde::{de, Deserialize, Deserializer};
+use serde::{Deserialize, Deserializer, de};
 
 use crate::{
-    ipfilter::IpFilter, KeySet, NoCipher, NtpClock, NtpPacket, NtpTimestamp, NtpVersion,
-    PacketParsingError, SystemSnapshot,
+    Cipher, KeySet, NtpClock, NtpPacket, NtpTimestamp, NtpVersion, PacketParsingError,
+    SystemSnapshot, ipfilter::IpFilter,
 };
 
 pub enum ServerAction<'a> {
@@ -94,7 +94,7 @@ pub struct Server<C> {
 
 // Quick estimation of ntp packet message version without doing full parsing
 fn fallback_message_version(message: &[u8]) -> u8 {
-    message.first().map(|v| (v & 0b0011_1000) >> 3).unwrap_or(0)
+    message.first().map_or(0, |v| (v & 0b0011_1000) >> 3)
 }
 
 impl<C> Server<C> {
@@ -164,6 +164,16 @@ impl<C> Server<C> {
     }
 }
 
+pub struct HandleInnerData<'a> {
+    pub action: ServerResponse,
+    pub reason: ServerReason,
+    pub version: NtpVersion,
+    pub nts: bool,
+    pub packet: NtpPacket<'a>,
+    pub cipher: Option<Box<dyn Cipher>>,
+    pub desired_size: Option<usize>,
+}
+
 impl<C: NtpClock> Server<C> {
     /// Handle a packet sent to the server
     ///
@@ -178,12 +188,55 @@ impl<C: NtpClock> Server<C> {
         buffer: &'a mut [u8],
         stats_handler: &mut impl ServerStatHandler,
     ) -> ServerAction<'a> {
-        let (mut action, mut reason) = self.intended_action(client_ip);
+        let HandleInnerData {
+            action,
+            reason,
+            version,
+            nts,
+            packet,
+            cipher,
+            desired_size,
+        } = match self.handle_inner(client_ip, recv_timestamp, message, stats_handler) {
+            Ok(value) => value,
+            Err(value) => return value,
+        };
 
+        let mut cursor = Cursor::new(buffer);
+        match packet.serialize(&mut cursor, &cipher.as_deref(), desired_size) {
+            Ok(_) => {
+                stats_handler.register(version.into(), nts, reason, action);
+                let length = cursor.position();
+                ServerAction::Respond {
+                    message: &cursor.into_inner()[..length as _],
+                }
+            }
+            Err(e) => {
+                tracing::debug!("Could not serialize response: {}", e);
+                stats_handler.register(
+                    version.into(),
+                    nts,
+                    ServerReason::InternalError,
+                    ServerResponse::Ignore,
+                );
+                ServerAction::Ignore
+            }
+        }
+    }
+
+    // FIXME: Figure out a way to split this
+    #[expect(clippy::too_many_lines)]
+    fn handle_inner<'a>(
+        &mut self,
+        client_ip: IpAddr,
+        recv_timestamp: NtpTimestamp,
+        message: &'a [u8],
+        stats_handler: &mut impl ServerStatHandler,
+    ) -> Result<HandleInnerData<'a>, ServerAction<'static>> {
+        let (mut action, mut reason) = self.intended_action(client_ip);
         if action == ServerResponse::Ignore {
             // Early exit for ignore
             stats_handler.register(fallback_message_version(message), false, reason, action);
-            return ServerAction::Ignore;
+            return Err(ServerAction::Ignore);
         }
 
         // Try and parse the message
@@ -197,7 +250,7 @@ impl<C: NtpClock> Server<C> {
                         ServerReason::ParseError,
                         ServerResponse::Ignore,
                     );
-                    return ServerAction::Ignore;
+                    return Err(ServerAction::Ignore);
                 }
             },
             Err(PacketParsingError::DecryptError(packet)) => {
@@ -215,7 +268,7 @@ impl<C: NtpClock> Server<C> {
                     ServerReason::ParseError,
                     ServerResponse::Ignore,
                 );
-                return ServerAction::Ignore;
+                return Err(ServerAction::Ignore);
             }
         };
 
@@ -230,7 +283,7 @@ impl<C: NtpClock> Server<C> {
                 ServerReason::Policy,
                 ServerResponse::Ignore,
             );
-            return ServerAction::Ignore;
+            return Err(ServerAction::Ignore);
         }
 
         let nts = cookie.is_some() || action == ServerResponse::NTSNak;
@@ -244,70 +297,71 @@ impl<C: NtpClock> Server<C> {
                     ServerReason::Policy,
                     ServerResponse::Ignore,
                 );
-                return ServerAction::Ignore;
-            } else {
-                action = ServerResponse::Deny;
-                reason = ServerReason::Policy;
+                return Err(ServerAction::Ignore);
             }
+            action = ServerResponse::Deny;
+            reason = ServerReason::Policy;
         }
 
-        let mut cursor = Cursor::new(buffer);
-        let result = match action {
-            ServerResponse::NTSNak => {
-                NtpPacket::nts_nak_response(packet).serialize(&mut cursor, &NoCipher, None)
-            }
+        let (packet, cipher, desired_size) = match action {
+            ServerResponse::NTSNak => (NtpPacket::nts_nak_response(packet), None, None),
             ServerResponse::Deny => {
                 if let Some(cookie) = cookie {
-                    NtpPacket::nts_deny_response(packet).serialize(
-                        &mut cursor,
-                        cookie.s2c.as_ref(),
-                        None,
-                    )
+                    (NtpPacket::nts_deny_response(packet), Some(cookie.s2c), None)
                 } else {
-                    NtpPacket::deny_response(packet).serialize(&mut cursor, &NoCipher, None)
+                    (NtpPacket::deny_response(packet), None, None)
                 }
             }
             ServerResponse::ProvideTime => {
                 if let Some(cookie) = cookie {
-                    NtpPacket::nts_timestamp_response(
-                        &self.system,
-                        packet,
-                        recv_timestamp,
-                        &self.clock,
-                        &cookie,
-                        &self.keyset,
-                    )
-                    .serialize(
-                        &mut cursor,
-                        cookie.s2c.as_ref(),
+                    (
+                        NtpPacket::nts_timestamp_response(
+                            &self.system,
+                            packet,
+                            recv_timestamp,
+                            &self.clock,
+                            &cookie,
+                            &self.keyset,
+                        ),
+                        Some(cookie.s2c),
                         Some(message.len()),
                     )
                 } else {
-                    NtpPacket::timestamp_response(&self.system, packet, recv_timestamp, &self.clock)
-                        .serialize(&mut cursor, &NoCipher, Some(message.len()))
+                    (
+                        NtpPacket::timestamp_response(
+                            &self.system,
+                            packet,
+                            recv_timestamp,
+                            &self.clock,
+                        ),
+                        None,
+                        Some(message.len()),
+                    )
                 }
             }
             ServerResponse::Ignore => unreachable!(),
         };
-        match result {
-            Ok(_) => {
-                stats_handler.register(version.into(), nts, reason, action);
-                let length = cursor.position();
-                ServerAction::Respond {
-                    message: &cursor.into_inner()[..length as _],
-                }
-            }
-            Err(e) => {
-                tracing::error!("Could not serialize response: {}", e);
-                stats_handler.register(
-                    version.into(),
-                    nts,
-                    ServerReason::InternalError,
-                    ServerResponse::Ignore,
-                );
-                ServerAction::Ignore
-            }
-        }
+
+        Ok(HandleInnerData {
+            action,
+            reason,
+            version,
+            nts,
+            packet,
+            cipher,
+            desired_size,
+        })
+    }
+
+    #[cfg(feature = "__internal-fuzz")]
+    pub fn fuzz_handle_inner<'a>(
+        &mut self,
+        client_ip: IpAddr,
+        recv_timestamp: NtpTimestamp,
+        message: &'a [u8],
+        stats_handler: &mut impl ServerStatHandler,
+    ) -> Result<HandleInnerData<'a>, ServerAction<'static>> {
+        self.handle_inner(client_ip, recv_timestamp, message, stats_handler)
     }
 }
 
@@ -433,12 +487,16 @@ impl<'de> Deserialize<'de> for IpSubnet {
 }
 
 #[cfg(test)]
+#[expect(
+    clippy::too_many_lines,
+    reason = "Long tests are not really a big problem"
+)]
 mod tests {
     use std::net::{Ipv4Addr, Ipv6Addr};
 
     use crate::{
-        nts_record::AeadAlgorithm, packet::AesSivCmac256, Cipher, DecodedServerCookie,
-        KeySetProvider, NtpDuration, NtpLeapIndicator, PollIntervalLimits,
+        Cipher, DecodedServerCookie, KeySetProvider, NoCipher, NtpDuration, NtpLeapIndicator,
+        PollIntervalLimits, nts::AeadAlgorithm, packet::AesSivCmac256,
     };
 
     use super::*;
